(*
BLAST is a tool for software model checking.
This file is part of BLAST.

Copyright (c) 2002-2007, The BLAST Team.
All rights reserved. 

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the authors nor their organizations 
   may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

(This is the Modified BSD License, see also
 http://www.opensource.org/licenses/bsd-license.php)

The BLAST Team consists of
        Dirk Beyer (SFU), Thomas A. Henzinger (EPFL),
        Ranjit Jhala (UCSD), and Rupak Majumdar (UCLA).

BLAST web page:
        http://mtc.epfl.ch/blast/

Bug reports:
        Dirk Beyer:      firstname.lastname@sfu.ca or
        Rupak Majumdar:  firstname@cs.ucla.edu or
        Ranjit Jhala:    lastname@cs.ucla.edu
 *)
(** Module for simplifying formula based on a set of known (non-)equalities,
    which can be extracted from a predicate which is assumed to be true.
    Contains in addition a simplification function that performs elementary
    simplifications *)
module Constraint_Simplifier :
sig
  
  type e = Ast.Expression.t

  (** Type to represent a set of known facts (facts that are
      assumed to be true *)
  type knowledge

  (** Type to represent a fact *)
  type fact =
      Eq of (e * e)
    | Ne of (e * e)

  (** Answer given when querying a knowledge *)
  type answ = Known_Eq | Known_Ne | Unknown

  (** Create a new, fresh set of known facts *)
  val new_knowledge : unit -> knowledge

  (** Add the provided facts to the set of known facts *)
  val learn : knowledge -> fact list -> unit

  (** Give a string representation of a set of known facts *)
  val string_of_knowledge : knowledge -> string

  (** Query the set of known facts to know what we know about two values *)
  val what_about : knowledge -> e -> e -> answ

  (** Remove implications that can be deduced from known facts *)
  val solve_facts : knowledge -> Ast.Predicate.predicate -> Ast.Predicate.predicate

  (** Return the list of facts that are implied by the given prediate *)
  val extract_facts : Ast.Predicate.predicate -> fact list

  (** Simplify a formula by extracting predicates, and then removing useless implications *)
  val extract_and_solve : Ast.Predicate.predicate -> Ast.Predicate.predicate

  (** Simplify a formula *)
  val simple_prune : Ast.Predicate.predicate -> Ast.Predicate.predicate

end
